!--------( PLSIM : Cross-section contour map of 3D image )--------
  implicit none
  integer,parameter :: KVER = 2018, LVER = 4
  integer :: lwkdir, lrtrim
  real,allocatable :: f(:,:), v(:,:), g(:,:), z(:), ds(:), dc(:)
  character(80) :: wdr, flog, fnin, fnam, buf
  character(72) :: out;  character(20) :: sdtm
  character(18) :: str2;  character(12) :: str1
  character(8) :: area, area1;  character(3) :: spl = '   '
  integer :: ldr, i, j, l, n, nhx, nhy, nly, kly, nver, nds
  integer :: nc, ncc, iorg, korg, ispa, ispb
  integer :: ixs, iys, mszx, mszy, mx, my
  integer :: kds, kpl, nlp, np, nsl, lcap
  integer :: nc1, ncc1, iorg1, korg1, ispa1, ispb1
  integer :: ixs1, iys1, mszx1, mszy1, mx1, my1, nly1, kly1, nlp1
  integer :: kdir, nkn, nke, nks, nkw, nsec
  real :: alt, dl, dll, dz, f1, f2, high, hlen, hscl
  real :: csz, step, thick, tmin, vmax, vmin, astp
  real :: vnul, vscl, vsum, wide, xs, ys, zmax, zmin, zt
  real :: vnul1, alt1

  call premsg('PLSIM : Cross-section contour map of 3D image')
  call opnpin()
  ldr = lwkdir(50,wdr) + 1
  flog = wdr;  fnin = wdr;  fnam = wdr
  call gparma('Enter LOG filename ==> ', 81-ldr, flog(ldr:80))
  call gparma('Enter Input AIMG data filename  ==> ', 81-ldr, fnin(ldr:80))
  open(10,file=fnin,status='old')
  n = 0;  nver = 0
  read(10,'(a)') buf
  do while (buf(1:1) == '#')
    if (buf(1:6) == '#AIMG:') then
      if (n == 0) then
        read(buf(7:80),*) kds, nds, nver, nhx, nhy
        if ((kds<0) .or. (kds>2) .or. (nds<1)) then
          call abendm('invalid AIMG file')
        endif
        l = (nds+7) / 8;  n = 1
        allocate(ds(nds));  allocate(dc(nds))

      else if (n < l) then
        read(buf(7:80),*) (ds(i),i=n*8-7,n*8)
        n = n + 1
      else if (n == l) then
        read(buf(7:80),*) (ds(i),i=n*8-7,nds)
        n = l + 1
      endif
    endif
    read(10,'(a)') buf
  enddo
  if ((nver/=KVER) .and. (nver/=LVER)) call abendm('AIMG Version mismatch')
  if (n /= (l+1)) call abendm('AIMG file header broken')
  if (buf(13:16) == '    ') then        !-- v2005
    read(buf,'(a8,i4,4x,4i8,8x,2i4)')   &
        area, ncc, iorg, korg, ispa, ispb, nly, kly
    if ((ncc<0) .or. (ncc>=400)) call abendm('invalid coord.no.')
    if (ncc < 200) then
      nc = ncc; ncc = ncc + 800
    else
      ncc = ncc - 200; nc = ncc
    endif
  else                                  !-- v2018
    read(buf,'(a8,4x,i4,4i8,8x,2i4)')   &
        area, ncc, iorg, korg, ispa, ispb, nly, kly
    nc = ncc
    if ((ncc >= 800) .and. (ncc < 1000)) nc = ncc - 800
  endif
  if ((nly /= nds) .or. (kly /= 0)) call abendm('AIMG file broken')
  if ((nc >= 1) .and. (nc <= 62)) then
    iorg = 0;  ispa = 0;  ispb = 0
    korg = (nc-30)*360 - 180
    if (nc > 60) korg = 0
    if (nc == 61) iorg = +5400
    if (nc == 62) iorg = -5400
  endif
  read(10,'(a)') buf
  read(buf,*) ixs, iys, mszx, mszy, mx, my, vnul, alt, nlp
  if (alt /= -1.) call abendm('invalid Alt. value')

  allocate(f(mx,my))

  xs = float(ixs) / 1000.;  ys = float(iys) / 1000.
  read(10,*) ((f(i,j),i=1,mx),j=1,my)

  write(out,'(a,a8,a,i4,4x,4i8)')   &
    '--- Area : ', area, '   Coord.', ncc, iorg,korg, ispa,ispb
  call premsg(out)
  write(out,'(14x,2a10,2x,2a8,2a6,a8)')   &
    'xs', 'ys', 'msz-x', 'msz-y', 'mx', 'my', 'Layers'
  call premsg(out)
  write(out,'(16x,2f10.2,2i8,3i6)') xs,ys, mszx,mszy, mx,my, nly
  call premsg(out)
  call gparmi('N-S section (0) or E-W section (1) ?  ==> ', kdir)
  if (kdir == 0) then
    call gparmi('Section seq.no. (1 - my) from W ?  ==> ', nsec)
    if ((nsec <= 0) .or. (nsec > my)) call abendm('illegal section no.')
    call premsg('  Specify widths to limit drawing range')
    call gparmi2('    No. of grids on S-bound, N-bound ==> ', nks,nkn)
    if ((nks < 0) .or. (nkn < 0)) call abendm('invalid value')
    if ((1+nks) >= (mx-nkn)) call abendm('no range')
    np = mx - nkn - nks;  hlen = float(mszx * (np-1))
    write(str1,'(a,i4)') 'S-N  iy#', nsec
    write(str2,'(a,i3,2x,a,i3,a)') '(LmS=', nks, 'LmN=', nkn, ')'

    allocate(z(np));  allocate(v(np,nly))

    do i=1,np;  z(i) = -f(i+nks,nsec);  enddo   !---  z():depth <== f():alt
  else if (kdir == 1) then
    call gparmi('Section seq.no. (1 - mx) from S ?  ==> ', nsec)
    if ((nsec <= 0) .or. (nsec > mx)) call abendm('illegal section no.')
    call premsg('  Specify widths to limit drawing range')
    call gparmi2('    No. of grids on W-bound, E-bound ==> ', nkw,nke)
    if ((nkw < 0) .or. (nke < 0)) call abendm('invalid value')
    if ((1+nkw) >= (my-nke)) call abendm('no range')
    np = my - nke - nkw;  hlen = float(mszy * (np-1))
    write(str1,'(a,i4)') 'W-E  ix#', nsec
    write(str2,'(a,i3,2x,a,i3,a)') '(LmW=', nks, 'LmE=', nkn, ')'

    allocate(z(np));  allocate(v(np,nly))

    do j=1,np;  z(j) = -f(nsec,j+nkw);  enddo   !---  z():depth <== f():alt
  else
    call abendm('invalid value')
  endif
  call gparma('Enter Output PS filename  ==> ', 81-ldr, fnam(ldr:80))
  if (fnam(ldr:ldr) == ' ') fnam(1:1) = char(0)
  if (fnam(80:80) /= ' ') call abendm('too long filename')

  do l=1,nly
    read(10,'(a)') buf
    do while (buf(1:1) == '#')
      read(10,'(a)') buf
    enddo
    if (buf(13:16) == '    ') then        !-- v2005
      read(buf,'(a8,i4,4x,4i8,8x,2i4)')   &
          area1, ncc1, iorg1, korg1, ispa1, ispb1, nly1, kly1
      if ((ncc1<0) .or. (ncc1>=400)) call abendm('invalid coord.no.')
      if (ncc1 < 200) then
        nc1 = ncc1; ncc1 = ncc1 + 800
      else
        ncc1 = ncc1 - 200; nc1 = ncc1
      endif
    else                                  !-- v2018
      read(buf,'(a8,4x,i4,4i8,8x,2i4)')   &
          area1, ncc1, iorg1, korg1, ispa1, ispb1, nly1, kly1
      nc1 = ncc1
      if ((ncc1 >= 800) .and. (ncc1 < 1000)) nc1 = ncc1 - 800
    endif
    if ((nly1 <= 0) .or. (kly1 /= l)) call abendm('AIMG file broken')
    read(10,'(a)') buf
    read(buf,*) ixs1, iys1, mszx1, mszy1, mx1, my1, vnul1, alt1, nlp1
    read(10,*) ((f(i,j),i=1,mx),j=1,my)
    if (kdir == 0) then
      do i=1,np
        if (f(i+nks,nsec) == vnul) then
          v(i,l) = vnul
        else
          v(i,l) = f(i+nks,nsec) / 100.
        endif
      enddo
    else
      do j=1,np
        if (f(nsec,j+nkw) == vnul) then
          v(j,l) = vnul
        else
          v(j,l) = f(nsec,j+nkw) / 100.
        endif
      enddo
    endif
  enddo
  close(10)

  n = -1
  do l=1,nly
    do i=1,np
      if (v(i,l) /= vnul) then
        n = 0;  vsum = 0.;  vmin = v(i,l);  vmax = v(i,l);  exit
      endif
    enddo
    if (n == 0) exit
  enddo
  if (n < 0) call abendm('no valid data')
  do l=1,nly; do i=1,np
    if (v(i,l) == vnul) cycle
    n = n + 1;  vsum = vsum + v(i,l)
    if (v(i,l) < vmin) vmin = v(i,l)
    if (v(i,l) > vmax) vmax = v(i,l)
  enddo; enddo
  vsum = vsum / float(n)
  zmin = z(1);  zmax = z(1)
  do i=2,np
    if (z(i) < zmin) zmin = z(i)
    if (z(i) > zmax) zmax = z(i)
  enddo
!---  zmin: depth of highest point of Terr Surf
!---  zmax: depth of lowest point of Terr Surf
  if (kds == 0) then
    zmax = ds(nly)
  else
    zmax = zmax + ds(nly)
  endif
!---  zmax: depth of deepest point of Bottom Layer bottom
  call premsg(' ')
  write(out,'(4x,a,f7.2,a,f7.2,a,f7.2)')   &
    'Data Range :  [', vmin, ' ,', vmax, ' ]    Average :', vsum
  call premsg(out)
  write(out,'(4x,a,f7.1,a,f7.1,a)')   &
    'Depth Range : [', zmin, ' ,', zmax, ' ] (m)'
  call premsg(out)
  call premsg('')
!---  kds   Layer config          bottom ds()  center dc()  ds(0)
!     0   Var.Depth Horizon         depth below Sea Level  -99999.
!     1   Const.Thick below Terr    depth below Terr Surf      0.
!---  2   Var.Thick below Terr      depth below Terr Surf      0,
  if (kds == 0) then
    tmin = ds(2) - ds(1)                !---  dc(1) undefined
  else
    tmin = ds(1);  dc(1) = ds(1) / 2.
  endif
  do l=2,nly
    thick = ds(l) - ds(l-1)
    if (thick < tmin) tmin = thick
    dc(l) = (ds(l-1)+ds(l)) / 2.
  enddo
  nsl = ifix((zmax-zmin)*3. / tmin) + 2
  dz = (zmax-zmin) / float(nsl-1)
!---  Define evenly spaced depth nodes for section mapping,
!      so that even the thinnest layer would be devided into 3.
!---    (total <nsl> nodes (including both ends) with the spacing <dz>)

  allocate(g(nsl,np))

  do i=1,np
    j = 0;  zt = zmin;  dl = z(i)
    do while (zt < dl)
      j = j + 1;  g(j,i) = vnul;  zt = zt + dz
    enddo
    if (kds == 0) then
      dl = (z(i)+ds(1)) / 2.
    else
      dl = z(i) + dc(1)
    endif
    do while (zt < dl)
      j = j + 1;  g(j,i) = v(i,1);  zt = zt + dz
    enddo
    do l=2,nly
      dll = dl
      if (kds == 0) then
        dl = dc(l)
      else
        dl = z(i) + dc(l)
      endif
      do while (zt < dl)
        f1 = zt - dll;  f2 = dl - zt;  j = j + 1
        if ((v(i,l-1) == vnul) .or. (v(i,l) == vnul)) then
          g(j,i) = vnul
          if (f1 == 0.) g(j,i) = v(i,l-1)
        else
          g(j,i) = (v(i,l-1)*f2 + v(i,l)*f1) / (f1 + f2)
        endif
        zt = zt + dz
      enddo
    enddo
    if (kds == 0) then
      dl = ds(nly)
    else
      dl = z(i) + ds(nly)
    endif
    do while (zt < dl)
      j = j + 1;  g(j,i) = v(i,nly);  zt = zt + dz
    enddo
    do while (j < nsl)
      j = j + 1;  g(j,i) = vnul
    enddo
  enddo

  call gparmi('  Select  1(portrait) or 2(landscape) ==> ', kpl)
  if (kpl == 1) spl = 'A4P'
  if (kpl == 2) spl = 'A4L'
  if (spl == '   ') call abendm('invalid value')
  call gparmf('  Width of Drawing (in cm) ?  ==> ', wide)
  if ((wide <= 0.) .or. (wide > 25.)) call abendm('invalid width')
  if ((kpl == 1) .and. (wide > 18.)) call abendm('invalid width')
  hscl = hlen * 100. / wide
  call gparmf('  Height of Drawing (in cm) ?  ==> ', high)
  if ((high <= 0.) .or. (high > 25.)) call abendm('invalid height')
  if ((kpl == 2) .and. (high > 18.)) call abendm('invalid height')
  vscl = (zmax-zmin) * 100. / high
  call premsg('  Specify contour interval (A/m)')
  call gparmf('   (minus to draw even crowded contour) ==> ', step)
  if (step == 0.) call abendm('invalid parameter')
  astp = abs(step);  lcap = 5
  if ((astp == 5.) .or. (astp == 0.5) .or. (astp == 0.05)) lcap = 4
  if ((astp == 2.5) .or. (astp == 0.25) .or. (astp == 0.025)) lcap = 4
  if (step < 0.) lcap = -lcap
  call premsg('  Char-size of contour value (in cm)')
  call gparmf('   (If zero (0.), no values drawn.)  ==> ', csz)
  if (csz < 0.) call abendm('invalid character size')
  call clspin()
  write(6,'(a)') '----------------------------------------'
  write(6,'(a)') 'PLSIM : Cross-section contour map of 3D image'
  write(6,'(a,a)') ' Input AIMG filename : ', fnin(1:lrtrim(fnin))
  write(6,'(a,a)') ' Output PS filename  : ', fnam(1:lrtrim(fnam))
  write(6,'(a,a8,a,i4,4x,4i8)')   &
    'Areaname : ', area, '   Coord.', ncc, iorg,korg, ispa,ispb
  write(6,'(14x,2a10,2x,2a8,2a6,a8)')   &
    'xs', 'ys', 'msz-x', 'msz-y', 'mx', 'my', 'Layers'
  write(6,'(16x,2f10.2,2i8,3i6)') xs,ys, mszx,mszy, mx,my, nly
  write(6,'(6x,a,a,4x,a,i4,4x,a)')   &
    'Section :  ', str1, 'np=', np, str2
  write(6,'(8x,a,f7.1,a,f7.1,a,4x,a,i4)')   &
    'Depth range : [', zmin, ' ,', zmax, ' ] (m)', 'nsl=', nsl
  write(6,'(6x,2a,4x,a,f8.0,4x,a,f8.0)')   &
    'Sheet: ', spl, 'H-scale  1 /', hscl, 'V-scale  1 /', vscl
  write(6,'(6x,a,f6.3,1x,a,4x,a,i3,4x,f8.1,a1,f8.1,a1)')   &
    'step=', astp, 'A/m', 'lcapt=', lcap, wide, 'W', high, 'H'
  write(6,'(8x,a,f6.2,a)') 'char.size of cont.value : ', csz, ' cm'

  call psopn(fnam, spl)
  call plots(1.5, 1.5)
  call lstyle('T', 0.4, 0., 0, -255)
  call ptext(area, 8,  -1., -1.5, 0)
  call ptext(fnin, 50, 2., -1.5, 0)
  call conts(0., high, wide, -high, -np, -nsl, csz)
  call contr(g, vnul, astp, lcap, -99.999, 0., 11)
  call contr(g, vnul, astp, lcap, -astp/2., 99.999, 10)
  call newpen(2)
  call wrect(0., 0., wide, high)
  call newpen(1)
  call ptext(str1, 12, 0., high+1., 0)
  call plote()
  call pscls()

  if (flog(ldr:ldr) /= ' ') then
    open(99,file=flog,access='append')
    write(99,'(//a)') 'PLSIM : Cross-section contour map of 3D image'
    write(99,'(a,a)') ' Input AIMG filename : ', fnin(1:lrtrim(fnin))
    write(99,'(a,a)') ' Output PS filename  : ', fnam(1:lrtrim(fnam))
    write(99,'(a,a8,a,i4,4x,4i8)')   &
      'Areaname : ', area, '   Coord.', ncc, iorg,korg,ispa,ispb
    write(99,'(14x,2a10,2x,2a8,2a6)')   &
      'xs', 'ys', 'msz-x', 'msz-y', 'mx', 'my'
    write(99,'(16x,2f10.2,2i8,2i6)') xs, ys, mszx, mszy, mx, my
    write(99,'(6x,a,a,4x,a,i4,4x,a)') 'Section :  ', str1, 'np=', np, str2
    write(99,'(8x,a,f7.1,a,f7.1,a,4x,a,i4)')   &
      'Depth range : [', zmin, ' ,', zmax, ' ] (m)', 'nsl=', nsl
    write(99,'(6x,2a,4x,a,f8.0,4x,a,f8.0)')   &
      'Sheet: ', spl, 'H-scale  1 /', hscl, 'V-scale  1 /', vscl
    write(99,'(6x,a,f6.3,1x,a,4x,a,i3,4x,f8.1,a1,f8.1,a1)')   &
      'step=', astp, 'A/m', 'lcapt=', lcap, wide, 'W', high, 'H'
    write(99,'(8x,a,f6.2,a)') 'char.size of cont.value : ', csz, ' cm'
    call strdtm(sdtm)
    write(99,'(a,a,a)') '=============== ', sdtm, '==============='
    close(99)
  endif
  stop
  end
